import * as Colors from "./Colors";
import { generateShades, ColorsInput, ShadesOptions } from "./Colors";

export interface ThemeInput {
  name: string;
  colors: ColorsInput;
}

const generateThemeOverrideClass = (input: {
  properties: any;
  className: string;
}) => {
  const propKeys = Object.keys(input.properties);

  if (propKeys.length > 0) {
    return ` 
    .${input.className} {${propKeys
      .map((key) => `\t--${key}: ${input.properties[key]};`)
      .join("\n")}}`;
  } else return "";
};

const generateThemeRoot = (input: { properties: any }) => {
  const propKeys = Object.keys(input.properties);

  if (propKeys.length > 0) {
    return `
    :root {${propKeys
      .map((key) => `\t--${key}: ${input.properties[key]};`)
      .join("\n")}}`;
  } else return "";
};

const generateTheme = (options: {
  output?: "CSS";
  themes: ThemeInput[];
  generateColorShades?: boolean | ShadesOptions;
}) => {
  if (
    options.generateColorShades !== undefined &&
    options.generateColorShades !== false
  ) {
    options.themes = options.themes.map((theme) => {
      theme.colors = {
        ...generateShades(
          theme.colors,
          typeof options.generateColorShades !== "boolean"
            ? options.generateColorShades
            : undefined
        ),
        ...theme.colors,
      };

      return theme;
    });
  }

  const themeRoot = generateThemeRoot({
    properties: { ...options.themes[0].colors },
  });

  if (options.themes.length > 1) {
    const themeOverrides = options.themes
      .slice(1)
      .map((theme) => {
        const overrideClass = generateThemeOverrideClass({
          properties: { ...theme.colors },
          className: theme.name,
        });

        return overrideClass;
      })
      .join("\n");

    return themeRoot + themeOverrides;
  } else return themeRoot;
};

export { generateThemeOverrideClass, generateThemeRoot, generateTheme };
export default {
  Colors,
  generateThemeOverrideClass,
  generateThemeRoot,
  generateTheme,
};
